Imports System.Xml
Public Class DatumsangabeListe
    Inherits List(Of IDatumsAngabe)
    Private m_LastExifZeitAngabe As DatumsAngabeExifZeit
    Private m_LastExaktAngabe As DatumsAngabeExakt
    Private m_LastMonatsAngabe As DatumsAngabeMonat
    Private m_LastJahrAngabe As DatumsAngabeJahr
    Private m_LastJahreszeitAngabe As DatumsAngabeJahreszeit
    Private m_LastJahrzehntAngabe As DatumsAngabeJahrzehnt

    Private m_Bilder As List(Of Bild)

    ReadOnly Property UnbekanntAngabe() As IDatumsAngabe
        Get
            Return UnbekanntDatumsAngabe
        End Get
    End Property
    ReadOnly Property LastExifZeitAngabe() As DatumsAngabeExifZeit
        Get
            If m_LastExifZeitAngabe Is Nothing Then
                Return New DatumsAngabeExifZeit(Date.Now)
            Else
                Return m_LastExifZeitAngabe.CreateClone
            End If
        End Get
    End Property
    ReadOnly Property LastExaktAngabe() As DatumsAngabeExakt
        Get
            If m_LastExaktAngabe Is Nothing Then
                Return New DatumsAngabeExakt(Date.Today)
            Else
                Return m_LastExaktAngabe.CreateClone
            End If
        End Get
    End Property
    ReadOnly Property LastMonatsAngabe() As DatumsAngabeMonat
        Get
            If m_LastMonatsAngabe Is Nothing Then
                Return New DatumsAngabeMonat(CType(Date.Today.Month, Monate), Date.Today.Year)
            Else
                Return m_LastMonatsAngabe.CreateClone
            End If
        End Get
    End Property
    ReadOnly Property LastJahrAngabe() As DatumsAngabeJahr
        Get
            If m_LastJahrAngabe Is Nothing Then
                Return New DatumsAngabeJahr(Date.Today.Year)
            Else
                Return m_LastJahrAngabe.CreateClone
            End If
        End Get
    End Property
    ReadOnly Property LastJahreszeitAngabe() As DatumsAngabeJahreszeit
        Get
            If m_LastJahreszeitAngabe Is Nothing Then
                Return New DatumsAngabeJahreszeit(Jahreszeiten.Frhling, Date.Today.Year)
            Else
                Return m_LastJahreszeitAngabe.CreateClone
            End If
        End Get
    End Property
    ReadOnly Property LastJahrzehntAngabe() As DatumsAngabeJahrzehnt
        Get
            If m_LastJahrzehntAngabe Is Nothing Then
                Return New DatumsAngabeJahrzehnt(Date.Today.Year)
            Else
                Return m_LastJahrzehntAngabe.CreateClone
            End If
        End Get
    End Property
    Public Sub New(ByVal Bilder As List(Of Bild))
        m_Bilder = Bilder
        Dim Ang As New DatumsAngabeUnbekannt
        UnbekanntDatumsAngabe = Ang
        Me.Add(Ang)
    End Sub
    Public Sub New(ByVal xNd As XmlNode, ByVal Bilder As List(Of Bild))
        m_Bilder = Bilder
        For Each Nd As XmlNode In xNd.ChildNodes
            If Nd.Name = "Unknown" Then
                Dim Ang As New DatumsAngabeUnbekannt(Nd)
                UnbekanntDatumsAngabe = Ang
                Me.Add(Ang)
            ElseIf Nd.Name = "DatumExakt" Then
                Me.Add(New DatumsAngabeExakt(Nd))
            ElseIf Nd.Name = "JahrMonat" Then
                Me.Add(New DatumsAngabeMonat(Nd))
            ElseIf Nd.Name = "Jahreszeit" Then
                Me.Add(New DatumsAngabeJahreszeit(Nd))
            ElseIf Nd.Name = "Jahr" Then
                Me.Add(New DatumsAngabeJahr(Nd))
            ElseIf Nd.Name = "Jahrzehnt" Then
                Me.Add(New DatumsAngabeJahrzehnt(Nd))
            ElseIf Nd.Name = "DatumExifZeit" Then
                Me.Add(New DatumsAngabeExifZeit(Nd))
            End If
        Next
    End Sub
    Public Sub Write_XML(ByVal writer As XmlTextWriter)
        For Each Ang As IDatumsAngabe In Me
            Ang.Write_XML(writer)
        Next
    End Sub
    Public Sub SetDatumForBild(ByVal SearchedValue As IDatumsAngabe, ByVal UsedBild As Bild)


        Dim SDA As IDatumsAngabe = Nothing
        For Each DA As IDatumsAngabe In Me
            If DA.IstGleich(SearchedValue) Then
                SDA = DA
                Exit For
            End If
        Next

        If SDA Is Nothing Then
            Me.Add(SearchedValue)
            SDA = SearchedValue
        End If

        Dim OldDA As IDatumsAngabe = UsedBild.Datumsangabe
        UsedBild.Datumsangabe = SDA

        If Not OldDA Is UnbekanntAngabe Then
            If Not OldDA Is SDA Then
                RemoveUnusedDatumsAngabe(OldDA)
            End If
        End If

        Select Case SDA.Typ
            Case DatumsangabeTypen.Exakt
                m_LastExaktAngabe = SDA
            Case DatumsangabeTypen.Jahr
                m_LastJahrAngabe = SDA
            Case DatumsangabeTypen.Jahreszeit
                m_LastJahreszeitAngabe = SDA
            Case DatumsangabeTypen.Jahrzehnt
                m_LastJahrzehntAngabe = SDA
            Case DatumsangabeTypen.Monat
                m_LastMonatsAngabe = SDA
            Case DatumsangabeTypen.ExifZeit
                m_LastExifZeitAngabe = SDA
        End Select

    End Sub
    Private Sub RemoveUnusedDatumsAngabe(ByVal AngabeToFree As IDatumsAngabe)
        Dim IsUsed As Boolean = False
        For Each Bld As Bild In m_Bilder
            If Bld.Datumsangabe Is AngabeToFree Then
                IsUsed = True
                Exit For
            End If
        Next
        If Not IsUsed Then
            Me.Remove(AngabeToFree)
        End If
    End Sub

End Class
