Imports System.ComponentModel
Public Class ZoomListView

    Public Event SelectedBilder_Geaendert()


#Region " Private Variablen"
    'Bitmap auf dem die gezeichnete Fläche des ZoomlistView-Controls, solange keine
    'Änderungen vorgenommen werden, gespeichert bleibt
    Private m_BMP As Bitmap

    'Erledigt Aufgaben zur Positionierung der ZoomListViewItems
    Private WithEvents m_Positioner As ZoomListViewPositioner

    'Die Original-Bildliste(unsortiert)
    Private m_BasisListe As List(Of Bild)

    'Eigentliche Bildliste, die bearbeitet wird, in die per Drag & Drop Bilder eingefügt werden
    'Entweder die Original-Bildliste oder sortierte Bildliste
    Private m_Bilder As List(Of Bild)

    'Auflistung der angezeigten ZoomListViewItems
    Private m_Items As New List(Of ZoomListViewItem)

    'ausgewählte ZoomListViewItems
    Private m_SelectedItems As New List(Of ZoomListViewItem)

    'Der Thread, der dafür sorgt, das nur auf der Bildfläche sichtbare Images
    'geladen werden, und das Ressourcen von nicht sichtbaren Images freigegeben 
    'werden. Damit wird das Arbeiten mit großen Bildlisten ermöglicht
    Private m_RefreshBitmapThread As Threading.Thread

    'wird gebraucht, um Textzeilen, die nicht vollständig angezeigt werden können, mit "..."
    'zu ergänzen
    Private m_EllepsisFormat As StringFormat

    'Immer wieder gebrauchter Pen, um das Selection-Rechteck zu zeichnen
    Private m_SelectionPen As Pen

    'Timer, der beim Ziehen eines ZoomListViewItems nach oben oder unten 
    'für ein automatisches Scrollen innerhalb des ZoomListView sorgt
    Private WithEvents m_MoveTimer As Timer

    'Einfügemarke
    Private m_InsertionMark As ZoomListViewInsertionMark

    'Event das notwendig ist, wenn ein ZoomListViewItem per Drag & Drop woandershin 
    'bewegt werden soll
    'Public Event ItemDrag(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemDragEventArgs)
    Public Event ItemDrag(ByVal sender As Object, ByVal e As ZoomListViewItemDragArgs)


    'gibt an, ob beim Drag & Drop aus dem Control herausgezogene Elemente gelöscht werden 
    Private m_AllowsRemoving As Boolean = False

    Private WithEvents MyMoveArgs As ZoomListViewItemDragArgs

    'Die angezeigten Bilder können nach verschiedenen Kriterien sortiert werden
    Private m_SortierTyp As BildSortierTypen = BildSortierTypen.Normal
#End Region


    Private Class ZoomListViewItemComparer
        Implements IComparer(Of ZoomListViewItem)
        Private m_Items As List(Of ZoomListViewItem)
        Public Sub New(ByVal Items As List(Of ZoomListViewItem))
            m_Items = Items
        End Sub
        Public Function Compare(ByVal x As ZoomListViewItem, ByVal y As ZoomListViewItem) As Integer Implements System.Collections.Generic.IComparer(Of ZoomListViewItem).Compare
            If m_Items.Contains(x) Then
                If m_Items.Contains(y) Then
                    Return m_Items.IndexOf(x).CompareTo(m_Items.IndexOf(y))
                Else
                    Return 1
                End If
            Else
                Return -1
            End If
        End Function
    End Class


    Public Class ZoomListViewItemDragArgs
        Private m_Source As Object
        Private m_Ziel As Object
        Private m_SelectedBilder() As Bild
        Public Event ErfolgreichAusgefuehrt()
        ReadOnly Property Source() As Object
            Get
                Return m_Source
            End Get
        End Property
        ReadOnly Property SelectedBilder() As Bild()
            Get
                Return m_SelectedBilder
            End Get
        End Property
        Property Ziel() As Object
            Get
                Return m_Ziel
            End Get
            Set(ByVal value As Object)
                m_Ziel = value
            End Set
        End Property
        Public Sub New(ByVal Source As Object, ByVal SelectedBilder() As Bild)
            m_Source = Source
            m_SelectedBilder = SelectedBilder
        End Sub
        Public Sub ErfolgMeldung()
            RaiseEvent ErfolgreichAusgefuehrt()
        End Sub
    End Class


#Region " Properties "
    ReadOnly Property SelectedBilder() As List(Of Bild)
        Get
            Dim Liste As New List(Of Bild)
            For Each Itm As ZoomListViewItem In m_SelectedItems
                Liste.Add(Itm.Bild)
            Next
            Return Liste
        End Get
    End Property
    'Private Sub ClearSelectedItems()
    '    If m_SelectedItems.Count > 0 Then
    '        m_SelectedItems.Clear()
    '        RaiseEvent SelectedBilder_Geaendert()
    '    End If
    'End Sub
    Private Sub SetSelectedItems(ByVal SelItems As List(Of ZoomListViewItem), ByVal ExtraInvalidateErforderlich As Boolean)
        Dim EventErforderlich As Boolean = False

        If SelItems.Count = 0 Then
            If m_SelectedItems.Count > 0 Then
                m_SelectedItems.Clear()
                EventErforderlich = True
            End If
        Else
            If m_SelectedItems.Count = 0 Then
                m_SelectedItems.AddRange(SelItems)
                EventErforderlich = True
            Else
                If m_SelectedItems.Count <> SelItems.Count Then
                    m_SelectedItems.Clear()
                    m_SelectedItems.AddRange(SelItems)
                    EventErforderlich = True
                Else
                    Dim Ungleich As Boolean = False
                    For I As Integer = 0 To m_SelectedItems.Count - 1
                        If m_SelectedItems(I) IsNot SelItems(I) Then
                            Ungleich = True
                            Exit For
                        End If
                    Next
                    If Ungleich Then
                        m_SelectedItems.Clear()
                        m_SelectedItems.AddRange(SelItems)
                        EventErforderlich = True
                    End If
                End If
            End If
        End If


        If EventErforderlich Then
            If ExtraInvalidateErforderlich Then
                m_BMP = Nothing
                Invalidate()
            End If
            RaiseEvent SelectedBilder_Geaendert()
        End If

    End Sub
    Private Sub AddSelectedItem(ByVal NewItem As ZoomListViewItem, ByVal ExtraInvalidateErforderlich As Boolean)
        If Not m_SelectedItems.Contains(NewItem) Then
            m_SelectedItems.Add(NewItem)
            If ExtraInvalidateErforderlich Then
                m_BMP = Nothing
                Invalidate()
            End If
            RaiseEvent SelectedBilder_Geaendert()
        End If
    End Sub
    Private Sub RemoveSelectedItem(ByVal DelItem As ZoomListViewItem, ByVal ExtraInvalidateErforderlich As Boolean)
        If m_SelectedItems.Contains(DelItem) Then
            m_SelectedItems.Remove(DelItem)
            If ExtraInvalidateErforderlich Then
                m_BMP = Nothing
                Invalidate()
            End If
            RaiseEvent SelectedBilder_Geaendert()
        End If
    End Sub


    'ReadOnly Property SelectedBild() As Bild
    '    Get
    '        If m_SelectedItem Is Nothing Then
    '            Return Nothing
    '        Else
    '            Return m_SelectedItem.Bild
    '        End If
    '    End Get
    'End Property

    <Category("Appearance"), Description("Kantenlänge der anzuzeigenden Thumbnails")> _
    Property PictureWidth() As Integer
        Get
            Return m_Positioner.PictureWidth
        End Get
        Set(ByVal value As Integer)
            If m_Positioner.PictureWidth <> value Then

                RemovePositionerHandler()
                AbortRefreshThread()

                m_Positioner.PictureWidth = value
                Positionen_Uebernehmen()
                Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
                Visibility_Einstellen()

                m_BMP = Nothing
                Invalidate()

                AddPositionerHandler()
                StartRefreshThread()

            End If
        End Set
    End Property

    <Category("Appearance"), Description("Sortierung für angezeigte Bilder")> _
    Property SortierTyp() As BildSortierTypen
        Get
            Return m_SortierTyp
        End Get
        Set(ByVal value As BildSortierTypen)
            If m_SortierTyp <> value Then



                m_SortierTyp = value
                RemovePositionerHandler()
                AbortRefreshThread()

                Dim SelectedBilder As New List(Of Bild)
                For Each Itm As ZoomListViewItem In m_SelectedItems
                    SelectedBilder.Add(Itm.Bild)
                Next

                m_Bilder = New List(Of Bild)
                m_Bilder.AddRange(m_BasisListe.ToArray)
                Select Case m_SortierTyp
                    Case BildSortierTypen.Autor
                        m_Bilder.Sort(New BildAutorComparer)
                    Case BildSortierTypen.Datum
                        m_Bilder.Sort(New BildDatumsComparer)
                    Case BildSortierTypen.Quelle
                        m_Bilder.Sort(New BildQuellenComparer)
                End Select

                For Each Itm As ZoomListViewItem In m_Items
                    Itm.Dispose()
                Next
                m_Items.Clear()

                Dim NewSelectedItems As New List(Of ZoomListViewItem)

                If m_Bilder IsNot Nothing Then
                    For Each Bld As Bild In m_Bilder
                        Dim Itm As New ZoomListViewItem(Me, Bld)
                        m_Items.Add(Itm)
                        If SelectedBilder.Contains(Bld) Then
                            NewSelectedItems.Add(Itm)
                        End If
                    Next
                End If


                m_Positioner.ItemCount = m_Items.Count
                Positionen_Uebernehmen()
                Me.AutoScrollPosition = New Point(0, 0)
                Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
                Visibility_Einstellen()

                SetSelectedItems(NewSelectedItems, False)

                m_BMP = Nothing
                Invalidate()

                AddPositionerHandler()
                StartRefreshThread()


            End If
        End Set
    End Property

    <Category("Appearance"), Description("Bildliste, die angezeigt werden soll")> _
    Property Bilder() As List(Of Bild)
        Get
            Return m_BasisListe
        End Get
        Set(ByVal value As List(Of Bild))
            Dim TmpListe As List(Of Bild)
            If value Is Nothing Then
                TmpListe = New List(Of Bild)
                Me.AllowDrop = False
            Else
                TmpListe = value
                Me.AllowDrop = True
            End If
            'If m_Bilder IsNot value Then
            RemovePositionerHandler()
            AbortRefreshThread()

            m_BasisListe = TmpListe


            m_Bilder = New List(Of Bild)
            m_Bilder.AddRange(m_BasisListe.ToArray)
            Select Case m_SortierTyp
                Case BildSortierTypen.Autor
                    m_Bilder.Sort(New BildAutorComparer)
                Case BildSortierTypen.Datum
                    m_Bilder.Sort(New BildDatumsComparer)
                Case BildSortierTypen.Quelle
                    m_Bilder.Sort(New BildQuellenComparer)
            End Select



            For Each Itm As ZoomListViewItem In m_Items
                Itm.Dispose()
            Next
            m_Items.Clear()
            If m_Bilder IsNot Nothing Then
                For Each Bld As Bild In m_Bilder
                    m_Items.Add(New ZoomListViewItem(Me, Bld))
                Next
            End If




            m_Positioner.ItemCount = m_Items.Count
            Positionen_Uebernehmen()
            Me.AutoScrollPosition = New Point(0, 0)
            Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
            Visibility_Einstellen()

            SetSelectedItems(New List(Of ZoomListViewItem), False)

            m_BMP = Nothing
            Invalidate()

            AddPositionerHandler()
            StartRefreshThread()
            'End If
        End Set
    End Property

    <Category("Appearance"), Description("Gibt an und legt fest, ob per Drag & Drop aus dem Control herausgezogene Bilder gelöscht werden sollen")> _
    Property AllowsRemoving() As Boolean
        Get
            Return m_AllowsRemoving
        End Get
        Set(ByVal value As Boolean)
            m_AllowsRemoving = value
        End Set
    End Property
#End Region

#Region " Constructor "
    Public Sub New()

        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()

        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserMouse, True)

        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_Positioner = New ZoomListViewPositioner(0, 100, Me.Font.Height, 10, Me.Size)
        m_EllepsisFormat = New System.Drawing.StringFormat
        m_EllepsisFormat.Trimming = StringTrimming.EllipsisCharacter

        m_SelectionPen = New Pen(SystemColors.Highlight, 4)

        m_MoveTimer = New Timer
        m_MoveTimer.Interval = 200
        m_MoveTimer.Start()
        AddHandler Me.ItemDrag, AddressOf X_ItemDrag
    End Sub
#End Region

#Region " Interne Funktionen "
    'Diese Prozedur legt nach jedem Scrollen oder Größenänderung des Controls fest,
    'welche ZoomListViewItems sichtbar sind und welche nicht,
    'um im Anschluss Speicher für nicht benötigte Images freizugeben bzw. benötigte 
    'Images wieder zu laden
    Private Sub Visibility_Einstellen()
        Dim AktuelleControlTopGrenze As Integer = Me.AutoScrollPosition.Y * -1
        Dim AktuelleControlBottomGrenze As Integer = AktuelleControlTopGrenze + Me.ClientSize.Height

        Dim PicVisibleHeight As Integer = m_Positioner.PictureWidth + Me.Font.Height * 2

        For Each Itm As ZoomListViewItem In m_Items
            If Itm.Location.Y <= AktuelleControlBottomGrenze AndAlso Itm.Location.Y + PicVisibleHeight >= AktuelleControlTopGrenze Then
                Itm.IsVisible = True
            Else
                Itm.IsVisible = False
            End If
        Next
    End Sub

    'Hier werden die Positionen, die im Positioner neu festgelegt wurden, auf die 
    'ZoomListViewItems übertragen
    Private Sub Positionen_Uebernehmen()
        If m_Items.Count > 0 Then
            For I As Integer = 0 To m_Items.Count - 1
                m_Items(I).Location = m_Positioner.ItemPositions(I)
            Next
        End If
    End Sub

    'Diese Funktion legt die Einfügemarke beim Drag & Drop fest
    Private Sub SetInsertionMark(ByVal Mark As ZoomListViewInsertionMark, ByVal InvalidateRequired As Boolean)
        Dim MustInvalidate As Boolean = False
        If Mark Is Nothing Then
            If m_InsertionMark IsNot Nothing Then
                m_InsertionMark = Nothing
                MustInvalidate = True
            End If
        Else
            If m_InsertionMark Is Nothing Then
                m_InsertionMark = Mark
                MustInvalidate = True
            Else
                If m_InsertionMark.Spalte <> Mark.Spalte OrElse m_InsertionMark.Zeile <> Mark.Zeile Then
                    m_InsertionMark = Mark
                    MustInvalidate = True
                End If
            End If
        End If
        If MustInvalidate Then
            If InvalidateRequired Then
                m_BMP = Nothing
                Invalidate()
            End If
        End If

    End Sub

    'Private Sub SetSelectedItem(ByVal NewItem As ZoomListViewItem)
    '    If m_SelectedItem IsNot NewItem Then
    '        m_SelectedItem = NewItem
    '        If m_SelectedItem Is Nothing Then
    '            RaiseEvent SelectedBild_Geaendert(Nothing)
    '        Else
    '            RaiseEvent SelectedBild_Geaendert(m_SelectedItem.Bild)
    '        End If
    '        m_BMP = Nothing
    '        Invalidate()
    '    End If
    'End Sub

#End Region

#Region " Paint "
    Protected Overrides Sub OnPaintBackground(ByVal e As System.Windows.Forms.PaintEventArgs)
        'Nichts notwendig
    End Sub

    'die folgende Prozedur ist die wahrscheinlich m häufigsten durchlaufene in einem Control.
    'Es ist also notwendig, genau an dieser Stelle möglichst auf zeit- und speicheraufwändige 
    'Prozeduren zu verzichten

    'Darum wird hier eine allgemein übliche Methode zum Zeichnen von Controls verwendet.
    'Alles notwendige wird auf ein Bitmap gezeichnet, das genau die Größe dieses Controls hat.
    'Solange sich an den anzuzeigenden Inhalten nichts ändert, wird dieses Bitmap jedes 
    'Mal bei OnPaint wieder gezeichnet.
    'Nur wenn der Inhalt des Controls oder seine Größe sich ändern, wird dieses Bitmap
    'gelöscht und muss dann hier wieder neu erstellt werden

    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)

        Dim gr As Graphics = pe.Graphics

        If IsNothing(m_BMP) Then
            m_BMP = New Bitmap(Me.Width, Me.Height)

            Dim g As Graphics = Graphics.FromImage(m_BMP)
            g.FillRectangle(Brushes.White, New Rectangle(0, 0, Me.Width, Me.Height))

            Dim Ft As Font = Me.Font
            Dim ThumbSize As New Size(m_Positioner.PictureWidth, m_Positioner.PictureWidth + Ft.Height * 2)
            Dim PicWidth As Integer = m_Positioner.PictureWidth
            Dim TopDist As Integer = Me.AutoScrollPosition.Y * -1

            For Each Itm As ZoomListViewItem In m_Items
                If Itm.IsVisible Then
                    Itm.Paint(g, ThumbSize, TopDist, Ft, m_EllepsisFormat, Ft.Height)
                    If m_SelectedItems.Contains(Itm) Then
                        g.DrawRectangle(m_SelectionPen, Itm.Location.X, Me.AutoScrollPosition.Y + Itm.Location.Y, PicWidth, PicWidth)
                    End If
                End If
            Next

            If m_InsertionMark IsNot Nothing Then
                m_InsertionMark.Paint(g, Me.AutoScrollPosition.Y * -1)
            End If
            g.Dispose()
        End If

        gr.DrawImage(m_BMP, 0, 0)

    End Sub
#End Region

#Region " PositionerHandler "

    'mit PositionerHandler wird hier der Eventhandler für m_Positioner.Changed bezeichnet
    'Der Positioner liefert Informationen zur Position der anzuzeigenden ZoomListViewItems
    'Um unnötige Schleifen zu vermeiden, soll dieser Handler benutzerdefiniert
    'hinzugefügt oder entfernt werden können.
    Private m_PositionerHandlerAdded As Boolean = False
    Private Sub RemovePositionerHandler()
        If m_PositionerHandlerAdded Then
            RemoveHandler m_Positioner.Changed, AddressOf m_Positioner_Changed
            m_PositionerHandlerAdded = False
        End If
    End Sub
    Private Sub AddPositionerHandler()
        If Not m_PositionerHandlerAdded Then
            AddHandler m_Positioner.Changed, AddressOf m_Positioner_Changed
            m_PositionerHandlerAdded = True
        End If
    End Sub
    Private Sub m_Positioner_Changed()
        RemovePositionerHandler()
        AbortRefreshThread()

        Visibility_Einstellen()
        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub


#End Region

#Region " Threads/Refreshing "

    'Der RefreshBitmapThread ist ein Thread, der dafür sorgt, dass Ressourcen für nicht angezeigte 
    'Images freigegeben werden und aktuell sichtbare Images neu geladen werden
    Private Sub AbortRefreshThread()
        If m_RefreshBitmapThread IsNot Nothing Then
            m_RefreshBitmapThread.Abort()
        End If
    End Sub
    Private Sub StartRefreshThread()
        If m_RefreshBitmapThread IsNot Nothing Then
            m_RefreshBitmapThread.Abort()
        End If
        m_RefreshBitmapThread = New Threading.Thread(AddressOf RefreshBitmaps)
        m_RefreshBitmapThread.IsBackground = True
        m_RefreshBitmapThread.Start()
    End Sub
    Private Sub RefreshBitmaps()
        For Each Itm As ZoomListViewItem In m_Items
            If Itm.IsVisible Then
                Itm.RefreshBitmap(m_Positioner.PictureWidth)
            Else
                Itm.DisposeBitmap()
            End If
            Application.DoEvents()
        Next
    End Sub

    Delegate Sub InvokeDelegate()
    Public Sub RefreshBitmap()
        'Diese Prozedur ist notwendig, wenn von einem fremden Thread aus (RefreshBitmapThread)
        'erzwungen werden soll, das die Oberfläche dieses Controls (nach Laden eines neuen Images)
        'neu gezeichnet wird.
        If InvokeRequired Then
            BeginInvoke(New InvokeDelegate(AddressOf InvokeMethod))
            Return
        End If
        InvokeMethod()
    End Sub
    Private Sub InvokeMethod()
        m_BMP = Nothing
        Invalidate()
    End Sub
#End Region

#Region " Resize/Scrolling "

    Private Sub PicZoomListView_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        RemovePositionerHandler()
        AbortRefreshThread()

        Dim AutoscrollFactor As Single = CSng(Me.AutoScrollPosition.Y * -1) / CSng(Me.AutoScrollMinSize.Height)
        m_Positioner.ControlSize = Me.Size
        Positionen_Uebernehmen()


        Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
        If Me.AutoScrollPosition.Y <> 0 Then
            Dim NeueTopPosition As Integer = Me.AutoScrollMinSize.Height * AutoscrollFactor
            If NeueTopPosition + Me.Size.Height > m_Positioner.NotwendigeHoehe Then
                NeueTopPosition = m_Positioner.NotwendigeHoehe - Me.Size.Height
            End If
            Me.AutoScrollPosition = New Point(0, NeueTopPosition)
        End If

        Visibility_Einstellen()

        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub

    Private Sub PicZoomListView_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles Me.Scroll
        RemovePositionerHandler()
        AbortRefreshThread()

        Visibility_Einstellen()
        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub
#End Region

#Region " DragDropScrolling "
    Private m_ScrollingUp As Boolean = False
    Private m_ScrollingDown As Boolean = False
    Private Sub AddScrollUp()
        If Not m_ScrollingUp Then
            AddHandler Me.m_MoveTimer.Tick, AddressOf ScrollUp
            m_ScrollingUp = True
        End If
    End Sub
    Private Sub RemoveScrollUp()
        If m_ScrollingUp Then
            RemoveHandler Me.m_MoveTimer.Tick, AddressOf ScrollUp
            m_ScrollingUp = False
        End If
    End Sub

    Private Sub AddScrollDown()
        If Not m_ScrollingDown Then
            AddHandler Me.m_MoveTimer.Tick, AddressOf ScrollDown
            m_ScrollingDown = True
        End If
    End Sub
    Private Sub RemoveScrollDown()
        If m_ScrollingDown Then
            RemoveHandler Me.m_MoveTimer.Tick, AddressOf ScrollDown
            m_ScrollingDown = False
        End If
    End Sub
    Private Sub ScrollUp(ByVal sender As Object, ByVal e As System.EventArgs)
        If Me.AutoScrollPosition.Y = 0 Then Exit Sub
        RemovePositionerHandler()
        AbortRefreshThread()

        Dim y As Integer = Me.AutoScrollPosition.Y * -1
        If y > m_Positioner.ZeilenHoehe Then
            y -= m_Positioner.ZeilenHoehe
        Else
            y = 0
        End If

        Me.AutoScrollPosition = New Point(0, y)
        Visibility_Einstellen()

        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub

    Private Sub ScrollDown(ByVal sender As Object, ByVal e As System.EventArgs)
        If Me.AutoScrollPosition.Y * -1 >= Me.AutoScrollMinSize.Height - Me.Height Then Exit Sub
        RemovePositionerHandler()
        AbortRefreshThread()

        Dim y As Integer = Me.AutoScrollPosition.Y * -1
        If y < Me.AutoScrollMinSize.Height - Me.Height - m_Positioner.ZeilenHoehe Then
            y += m_Positioner.ZeilenHoehe
        Else
            y = Me.AutoScrollMinSize.Height '- Me.Height - m_Positioner.ZeilenHoehe
        End If
        Me.AutoScrollPosition = New Point(0, y)

        Visibility_Einstellen()
        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub

#End Region

#Region " Mouse Events "




    Private Sub ZoomListView_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        RemoveScrollUp()
        RemoveScrollDown()

        Dim Pt As New Point(e.Location)
        Pt.Offset(0, Me.AutoScrollPosition.Y * -1)

        Dim SelIndex As Integer = m_Positioner.GetMouseHooveredIndex(Pt)

        If SelIndex > -1 Then
            Dim SelItem As ZoomListViewItem = m_Items(SelIndex)

            If ModifierKeys = Keys.Control Then
                If m_SelectedItems.Contains(SelItem) Then
                    RemoveSelectedItem(SelItem, True)
                Else
                    AddSelectedItem(SelItem, True)
                End If
            ElseIf ModifierKeys = Keys.Shift Then
                If m_SelectedItems.Count = 0 Then
                    AddSelectedItem(m_Items(SelIndex), True)
                Else
                    Dim FirstIndex As Integer = m_Items.Count
                    For Each Itm As ZoomListViewItem In m_SelectedItems
                        If m_Items.IndexOf(Itm) < FirstIndex Then
                            FirstIndex = m_Items.IndexOf(Itm)
                        End If
                    Next
                    Dim NeueListe As New List(Of ZoomListViewItem)
                    Dim IdxS, IdxE As Integer
                    If FirstIndex < SelIndex Then
                        IdxS = FirstIndex
                        IdxE = SelIndex
                    Else
                        IdxS = SelIndex
                        IdxE = FirstIndex
                    End If

                    For I As Integer = IdxS To IdxE
                        NeueListe.Add(m_Items(I))
                    Next
                    SetSelectedItems(NeueListe, True)
                End If
            Else
                If Not m_SelectedItems.Contains(SelItem) Then
                    Dim NeueListe As New List(Of ZoomListViewItem)
                    NeueListe.Add(SelItem)
                    SetSelectedItems(NeueListe, True)
                End If
            End If
        Else
            If ModifierKeys = Nothing Then
                SetSelectedItems(New List(Of ZoomListViewItem), True)
            End If
        End If


        'If SelIndex > -1 Then
        '    If ModifierKeys = Keys.Control Then
        '        If m_SelectedItems.Contains(m_Items(SelIndex)) Then
        '            RemoveSelectedItem(m_Items(SelIndex), True)
        '        Else
        '            AddSelectedItem(m_Items(SelIndex), True)
        '        End If
        '    ElseIf ModifierKeys = Keys.Shift Then
        '        If m_SelectedItems.Count = 0 Then
        '            AddSelectedItem(m_Items(SelIndex), True)
        '        Else
        '            Dim FirstIndex As Integer = m_Items.Count
        '            For Each Itm As ZoomListViewItem In m_SelectedItems
        '                If m_Items.IndexOf(Itm) < FirstIndex Then
        '                    FirstIndex = m_Items.IndexOf(Itm)
        '                End If
        '            Next
        '            Dim NeueListe As New List(Of ZoomListViewItem)
        '            For I As Integer = FirstIndex To SelIndex
        '                NeueListe.Add(m_Items(I))
        '            Next
        '            SetSelectedItems(NeueListe, True)
        '        End If
        '    End If
        '    'Dim SelItem As ZoomListViewItem = m_Items(SelIndex)
        '    'SetSelectedItem(SelItem)
        'Else
        '    SetSelectedItems(New List(Of ZoomListViewItem), True)
        '    'SetSelectedItem(Nothing)
        'End If
    End Sub

    Private Sub ZoomListView_MouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseWheel
        RemovePositionerHandler()
        AbortRefreshThread()

        Visibility_Einstellen()

        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub
    Private Sub ZoomListView_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        RemoveScrollUp()
        RemoveScrollDown()
        SetInsertionMark(Nothing, True)
    End Sub
    Private Sub ZoomListView_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove

        If e.Button <> Windows.Forms.MouseButtons.Left Then
            RemoveScrollUp()
            RemoveScrollDown()
            Exit Sub
        End If

        If e.Y < 0 Then
            RemoveScrollDown()
            AddScrollUp()
        ElseIf e.Y > Me.ClientSize.Height Then
            RemoveScrollUp()
            AddScrollDown()
        Else
            RemoveScrollUp()
            RemoveScrollDown()
        End If

        If m_SelectedItems.Count > 0 Then
            m_SelectedItems.Sort(New ZoomListViewItemComparer(m_Items))
            Dim Bilder(m_SelectedItems.Count - 1) As Bild
            For I As Integer = 0 To m_SelectedItems.Count - 1
                Bilder(I) = m_SelectedItems(I).Bild
            Next

            MyMoveArgs = New ZoomListViewItemDragArgs(Me, Bilder)
            RaiseEvent ItemDrag(Me, MyMoveArgs)
        Else
            MyMoveArgs = Nothing
        End If
    End Sub

    Private Sub ZoomListView_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.MouseLeave
        Me.SetInsertionMark(Nothing, True)
    End Sub
#End Region

#Region " DragDrop "
    Public Overrides Property AllowDrop() As Boolean
        Get
            Return MyBase.AllowDrop
        End Get
        Set(ByVal value As Boolean)
            MyBase.AllowDrop = value
            If value Then
                'AddHandler Me.ItemDrag, AddressOf X_ItemDrag
                AddHandler Me.DragEnter, AddressOf X_DragEnter
                AddHandler Me.DragOver, AddressOf X_DragOver
                AddHandler Me.DragLeave, AddressOf X_DragLeave
                AddHandler Me.DragDrop, AddressOf X_DragDrop
            Else
                'RemoveHandler Me.ItemDrag, AddressOf X_ItemDrag
                RemoveHandler Me.DragEnter, AddressOf X_DragEnter
                RemoveHandler Me.DragOver, AddressOf X_DragOver
                RemoveHandler Me.DragLeave, AddressOf X_DragLeave
                RemoveHandler Me.DragDrop, AddressOf X_DragDrop
            End If
        End Set
    End Property
    'Startet die Drag & Drop-Aktion, wenn ein Item bewegt wird
    Private Sub X_ItemDrag(ByVal sender As Object, ByVal e As ZoomListViewItemDragArgs)
        Me.DoDragDrop(e, DragDropEffects.Move)
    End Sub



    'Legt den Ziel-Drop-Effekt fest
    Private Sub X_DragEnter(ByVal sender As Object, ByVal e As DragEventArgs)
        RemoveScrollUp()
        RemoveScrollDown()
        e.Effect = e.AllowedEffect
    End Sub
    'Verschiebt die Einfügemarke, wenn ein Item bewegt wird
    Private Sub X_DragOver(ByVal sender As Object, ByVal e As DragEventArgs)
        RemoveScrollUp()
        RemoveScrollDown()

        If m_SortierTyp = BildSortierTypen.Normal Then
            Dim targetPoint As Point = Me.PointToClient(New Point(e.X, e.Y))
            targetPoint.Offset(0, Me.AutoScrollPosition.Y * -1)
            Dim Pos As ZoomListViewInsertionMark = m_Positioner.GetMoveTargetIndex(targetPoint)
            SetInsertionMark(Pos, True)
        End If

    End Sub

    ' Entfernt die Einfügemarke, wenn die Maus das Control verlässt
    Private Sub X_DragLeave(ByVal sender As Object, ByVal e As EventArgs)
        Dim MousePos As Point = Me.PointToClient(Control.MousePosition)
        If MousePos.Y < 0 Then
            AddScrollUp()
        ElseIf MousePos.Y > Me.ClientSize.Height Then
            AddScrollDown()
        End If
        SetInsertionMark(Nothing, True)
    End Sub

    'Führt die Drag & Drop-Aktion beim Loslassen des Mausbuttons aus
    Private Sub X_DragDrop(ByVal sender As Object, ByVal e As DragEventArgs)
        RemoveScrollUp()
        RemoveScrollDown()



        If Not e.Data.GetDataPresent(GetType(ZoomListViewItemDragArgs)) Then
            SetInsertionMark(Nothing, True)
            Exit Sub
        End If

        Dim obj As Object = CType(e.Data.GetData(GetType(ZoomListViewItemDragArgs)), System.Object)
        Dim Args As ZoomListViewItemDragArgs = CType(obj, ZoomListViewItemDragArgs)

        If Args.Source Is Me AndAlso m_SortierTyp <> BildSortierTypen.Normal Then
            'Verschiebung innerhalb der eigenen Liste
            'ist bei sortierter Anzeige nicht möglich
            Exit Sub
        End If

        'Danach bleiben jetzt drei Möglichkeiten übrig:
        'Verschiebung innerhalb der eigenen (Args.Source Is Me) unsortierten Liste
        'Verschiebung in eine fremde (Args.Source IsNot Me) sortierte Liste
        'Verschiebung in eine fremde unsortierte Liste

        'Bei Verschiebung in eine unsortierte Liste spielt die Position von 
        'InsertionMark eine Rolle. (Fall(1))  Alle Items werden an dieser Stelle untergebracht.
        'Bei Verschiebung in eine sortierte Liste werden die Items 
        'der Basis-Liste angefügt und nachher neu sortiert (Fall(2))

        'm_Bilder ist immer eine (ggf.sortierte) Kopie von m_BasisListe
        'm_BasisListe ist eine ggf. auch woanders an Daten gebundene (List(of Bild))

        'Fall(1)
        If m_SortierTyp = BildSortierTypen.Normal Then
            Abort_AllThumbCheckThread()
            RemovePositionerHandler()
            AbortRefreshThread()


            Dim targetIndex As Integer = m_InsertionMark.ZielIndex

            Dim ItemsToMove As New List(Of ZoomListViewItem)
            For Each Bld As Bild In Args.SelectedBilder
                Dim FoundItem As ZoomListViewItem = Nothing
                For Each Itm As ZoomListViewItem In m_Items
                    If Itm.Bild Is Bld Then
                        FoundItem = Itm
                        Exit For
                    End If
                Next
                If FoundItem Is Nothing Then
                    ItemsToMove.Add(New ZoomListViewItem(Me, Bld))
                Else
                    ItemsToMove.Add(FoundItem)
                End If
            Next


            'Jetzt sind die MoveItems sortiert.
            'Der Zielindex kann aber zwischen oder nach den MoveItems liegen

            Dim TmpItem As New ZoomListViewItem
            m_Items.Insert(targetIndex, TmpItem)
            For Each Itm As ZoomListViewItem In ItemsToMove
                If m_Items.Contains(Itm) Then
                    m_Items.Remove(Itm)
                    m_Bilder.Remove(Itm.Bild)
                    m_BasisListe.Remove(Itm.Bild)
                End If
            Next
            Dim Idx As Integer = m_Items.IndexOf(TmpItem)
            m_Items.Remove(TmpItem)
            Dim Zahl As Integer = 0
            For Each Itm As ZoomListViewItem In ItemsToMove
                m_Items.Insert(Idx + Zahl, Itm)
                m_Bilder.Insert(Idx + Zahl, Itm.Bild)
                m_BasisListe.Insert(Idx + Zahl, Itm.Bild)
                Zahl += 1
            Next

            m_Positioner.ItemCount = m_Items.Count
            Positionen_Uebernehmen()
            Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
            Visibility_Einstellen()

            SetInsertionMark(Nothing, False)

            SetSelectedItems(ItemsToMove, False)
            m_BMP = Nothing
            Invalidate()

            AddPositionerHandler()
            StartRefreshThread()
            Start_AllThumbCheckThread()
            Args.Ziel = Me
            Args.ErfolgMeldung()
        Else
            'Fall(2)
            'm_SortierTyp <> BildSortierTypen.Normal

            Abort_AllThumbCheckThread()
            RemovePositionerHandler()
            AbortRefreshThread()

            For Each bld As Bild In Args.SelectedBilder
                If Not m_BasisListe.Contains(bld) Then
                    m_BasisListe.Add(bld)
                End If
            Next

            m_Bilder = New List(Of Bild)
            m_Bilder.AddRange(m_BasisListe.ToArray)
            Select Case m_SortierTyp
                Case BildSortierTypen.Autor
                    m_Bilder.Sort(New BildAutorComparer)
                Case BildSortierTypen.Datum
                    m_Bilder.Sort(New BildDatumsComparer)
                Case BildSortierTypen.Quelle
                    m_Bilder.Sort(New BildQuellenComparer)
            End Select

            For Each Itm As ZoomListViewItem In m_Items
                Itm.Dispose()
            Next
            m_Items.Clear()

            Dim NewSelectedItems As New List(Of ZoomListViewItem)
            Dim SelBilder As New List(Of Bild)
            SelBilder.AddRange(Args.SelectedBilder)


            If m_Bilder IsNot Nothing Then
                For Each Bld As Bild In m_Bilder
                    Dim Itm As New ZoomListViewItem(Me, Bld)
                    m_Items.Add(Itm)
                    If SelBilder.Contains(Bld) Then
                        NewSelectedItems.Add(Itm)
                    End If
                Next
            End If


            m_Positioner.ItemCount = m_Items.Count
            Positionen_Uebernehmen()
            'Me.AutoScrollPosition = New Point(0, 0)
            Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
            Me.AutoScrollPosition = New Point(0, m_Positioner.GetAutoScrollPosition(m_Items.IndexOf(NewSelectedItems(0))))


            Visibility_Einstellen()

            SetSelectedItems(NewSelectedItems, False)
            SetInsertionMark(Nothing, False)

            m_BMP = Nothing
            Invalidate()


            AddPositionerHandler()
            StartRefreshThread()
            Start_AllThumbCheckThread()
            Args.Ziel = Me
            Args.ErfolgMeldung()
        End If

    End Sub

#End Region


    Private Sub MyMoveArgs_ErfolgreichAusgefuehrt() Handles MyMoveArgs.ErfolgreichAusgefuehrt
        'Dieses Event wurde "gefeuert", wenn ausgewählte Items erfolgreich per Drag & Drop gezogen wurden

        'Mit diesem Handler werden die entsprechenden Items(wenn erlaubt)
        'entfernt

        '(Aus der Basis-Liste können in dieser Anwendung in keinem Fall
        'Bilder per Drag&Drop herausgezogen werden.)
        If m_BasisListe Is MainBildListe Then Exit Sub
        If m_AllowsRemoving Then
            If MyMoveArgs.Ziel IsNot Me Then
                Dim BilderToRemove As New List(Of Bild)
                BilderToRemove.AddRange(MyMoveArgs.SelectedBilder)
                RemoveAllSelectedItems(BilderToRemove)
            End If
        End If

    End Sub
    Public Sub RemoveAllSelectedItems(ByVal BilderToRemove As List(Of Bild))
        'Diese Prozedur wird ausgeführt
        'a) nach Drag&Drop
        'b) beim Löschen aus einer Bildliste (die nicht die MainBildListe ist)
        Abort_AllThumbCheckThread()
        RemovePositionerHandler()
        AbortRefreshThread()



        Dim ItemsToRemove As New List(Of ZoomListViewItem)
        For Each Itm As ZoomListViewItem In m_Items
            If BilderToRemove.Contains(Itm.Bild) Then
                ItemsToRemove.Add(Itm)
            End If
        Next


        Dim FirstSelectedIndex As Integer = m_Items.IndexOf(ItemsToRemove(0))

        For Each Itm As ZoomListViewItem In ItemsToRemove
            m_Items.Remove(Itm)
            Itm.Dispose()
        Next

        For Each Bld As Bild In BilderToRemove
            m_Bilder.Remove(Bld)
            m_BasisListe.Remove(Bld)
        Next

        m_Positioner.ItemCount = m_Items.Count
        Positionen_Uebernehmen()
        Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)

        If FirstSelectedIndex > m_Items.Count - 1 Then
            FirstSelectedIndex = m_Items.Count - 1
        End If

        Me.AutoScrollPosition = New Point(0, m_Positioner.GetAutoScrollPosition(FirstSelectedIndex))


        Visibility_Einstellen()

        SetSelectedItems(New List(Of ZoomListViewItem), False)
        'SetInsertionMark(Nothing, False)

        m_BMP = Nothing
        Invalidate()


        AddPositionerHandler()
        StartRefreshThread()
        Start_AllThumbCheckThread()
    End Sub

    Public Sub RemoveItem(ByVal Item As ZoomListViewItem)
        'Diese Prozedur wird nach Bild.RemoveMeFromControls ausgeführt
        'und soll dazu dienen, dass ein Bild vor dem endgültigen 
        'Löschen aus allen abhängigen Controls und Listen
        'entfernt wird.

        RemovePositionerHandler()
        AbortRefreshThread()

        m_Items.Remove(Item)
        Item.Dispose()
        If m_BasisListe.Contains(Item.Bild) Then
            m_BasisListe.Remove(Item.Bild)
        End If

        If m_Bilder.Contains(Item.Bild) Then
            m_Bilder.Remove(Item.Bild)
        End If
        If m_SelectedItems.Contains(Item) Then
            RemoveSelectedItem(Item, False)
        End If


        m_Positioner.ItemCount = m_Items.Count
        Positionen_Uebernehmen()
        Me.AutoScrollMinSize = New Size(0, m_Positioner.NotwendigeHoehe)
        Visibility_Einstellen()


        m_BMP = Nothing
        Invalidate()

        AddPositionerHandler()
        StartRefreshThread()
    End Sub


End Class


