Public Class ZoomListViewItem
    Implements IDisposable

#Region " Private Variablen "
    'Das anzuzeigende Bild
    Private WithEvents m_Bild As Bild

    'Position des ZoomListViewItem innerhalb des ZoomListView-Controls
    Private m_Location As Point = New Point(0, 0)

    'Breite und Hhe des Thumbnails
    Private MyThumbWidth As Integer = -1

    'Temporr gespeicherte gezoomte Bitmap
    Private m_ZoomedBMP As Bitmap

    'Anzeigerechteck fr Bitmap innerhalb des ZoomListViewItem
    Private m_ZoomedBMPRect As Rectangle = Rectangle.Empty

    'wenn sich das ZoomListViewItem innerhalb des ScrollableControls 
    'im sichtbaren Bereich befindet True, ansonsten False
    Private m_IsVisible As Boolean = False

    'Referenz zum Parent-ZoomListView
    Private m_ZoomListView As ZoomListView
#End Region

#Region " Properties "
    ReadOnly Property Bild() As Bild
        Get
            Return m_Bild
        End Get
    End Property
    Property Location() As Point
        Get
            Return m_Location
        End Get
        Set(ByVal value As Point)
            m_Location = value
        End Set
    End Property
    Property IsVisible() As Boolean
        Get
            Return m_IsVisible
        End Get
        Set(ByVal value As Boolean)
            m_IsVisible = value
        End Set
    End Property
    ReadOnly Property Parent() As ZoomListView
        Get
            Return m_ZoomListView
        End Get
    End Property
#End Region

#Region " Constructor "
    Public Sub New()

    End Sub
    Public Sub New(ByVal Parent As ZoomListView, ByVal Bld As Bild)
        m_ZoomListView = Parent
        m_Bild = Bld
    End Sub
#End Region

#Region " Paint "
    Public Sub Paint(ByVal g As Graphics, ByVal ThumbSize As Size, ByVal TopDistance As Integer, ByVal Ft As Font, ByVal fmt As StringFormat, ByVal ZeilenHoehe As Integer)

        Dim PosY As Integer = m_Location.Y - TopDistance

        Dim InnerRect As New Rectangle(m_Location.X, PosY, ThumbSize.Width, ThumbSize.Width)
        Dim InnerBrush As New Drawing2D.LinearGradientBrush(InnerRect, Color.White, Color.LightGray, Drawing2D.LinearGradientMode.ForwardDiagonal)
        g.FillRectangle(InnerBrush, InnerRect)


        If m_ZoomedBMPRect <> Rectangle.Empty AndAlso m_ZoomedBMP IsNot Nothing Then
            g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias
            g.DrawImage(m_ZoomedBMP, New Rectangle(m_Location.X + m_ZoomedBMPRect.X, PosY + m_ZoomedBMPRect.Y, m_ZoomedBMPRect.Width, m_ZoomedBMPRect.Height))
            g.SmoothingMode = Drawing2D.SmoothingMode.Default
        End If

        g.DrawString(GetBeschreibungString, Ft, Brushes.Black, New RectangleF(1 + m_Location.X, PosY + ThumbSize.Width + 2, ThumbSize.Width - 2, ZeilenHoehe), fmt)
        g.DrawString(GetDatumsAngabeString, Ft, Brushes.Gray, New RectangleF(1 + m_Location.X, PosY + ThumbSize.Width + 2 + ZeilenHoehe, ThumbSize.Width - 2, ZeilenHoehe), fmt)

        g.DrawRectangle(Pens.LightGray, m_Location.X, PosY, ThumbSize.Width - 1, ThumbSize.Width - 1)

    End Sub
    Private Function GetBeschreibungString() As String
        Dim Beschr As String = ""

        Beschr = Me.m_Bild.QuellIndex.Trim
        If m_Bild.Beschreibung.Trim <> "" Then
            If Beschr <> "" Then Beschr &= " - "
            Beschr &= m_Bild.Beschreibung.Trim
        End If

        If Beschr <> "" Then
            Return Beschr
        Else
            Return m_Bild.Pfad
        End If
    End Function
    Private Function GetDatumsAngabeString() As String
        Dim ZStr As String = m_Bild.Datumsangabe.ToString
        If ZStr = "unbekannt" Then
            Return "-"
        Else
            Return ZStr
        End If
    End Function
#End Region

#Region " Refreshing "
    Public Sub RefreshBitmap(ByVal NewThumbWidth As Integer)

        If MyThumbWidth = NewThumbWidth Then Exit Sub
        MyThumbWidth = -1

        If IO.File.Exists(m_Bild.ThumbFileName) Then
            Dim Img As Image = Image.FromFile(m_Bild.ThumbFileName)
            m_ZoomedBMPRect = GetPictureRectangle(New Rectangle(1, 1, NewThumbWidth - 2, NewThumbWidth - 2), Img.Size)

            Dim Tmp_Bitmap As New Bitmap(m_ZoomedBMPRect.Width, m_ZoomedBMPRect.Height)
            Dim g As Graphics = Graphics.FromImage(Tmp_Bitmap)
            g.DrawImage(Img, New Rectangle(0, 0, m_ZoomedBMPRect.Width, m_ZoomedBMPRect.Height))

            Img.Dispose()
            m_ZoomedBMP = Tmp_Bitmap
            g.Dispose()
            GC.Collect()
        End If

        MyThumbWidth = NewThumbWidth
        m_ZoomListView.RefreshBitmap()

    End Sub

    Private Sub m_Bild_Changed() Handles m_Bild.Changed
        If Me.IsVisible Then
            RefreshBitmap(m_ZoomListView.PictureWidth)
        End If
    End Sub

    Private Sub m_Bild_RemoveMeFromControls() Handles m_Bild.RemoveMeFromControls
        m_ZoomListView.RemoveItem(Me)
    End Sub

    'aaa
    'Private Sub m_Bild_RemoveMeFromControls() Handles m_Bild.RemoveMeFromControls
    '    m_ZoomListView.RemoveItemBeforeDelete(Me)
    'End Sub
    Private Sub m_Bild_ThumbChanged() Handles m_Bild.ThumbChanged
        If Me.IsVisible Then
            MyThumbWidth = -1
            RefreshBitmap(m_ZoomListView.PictureWidth)
        End If
    End Sub
#End Region

#Region " Disposing "
    Public Sub DisposeBitmap()
        m_ZoomedBMPRect = Rectangle.Empty
        MyThumbWidth = -1
        If m_ZoomedBMP IsNot Nothing Then
            m_ZoomedBMP.Dispose()
            m_ZoomedBMP = Nothing
        End If
    End Sub

    Private disposedValue As Boolean = False        ' So ermitteln Sie berflssige Aufrufe

    ' IDisposable
    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                ' TODO: Nicht verwaltete Ressourcen freigeben, wenn sie explizit aufgerufen werden
                If m_ZoomedBMP IsNot Nothing Then
                    m_ZoomedBMP.Dispose()
                End If
            End If

            ' TODO: Gemeinsam genutzte nicht verwaltete Ressourcen freigeben
        End If
        Me.disposedValue = True
    End Sub

#Region " IDisposable Support "
    ' Dieser Code wird von Visual Basic hinzugefgt, um das Dispose-Muster richtig zu implementieren.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' ndern Sie diesen Code nicht. Fgen Sie oben in Dispose(ByVal disposing As Boolean) Bereinigungscode ein.
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region
#End Region

End Class


