Imports System.Drawing.Imaging
Public Class frm_EditColors
    Private m_BaseImage As Image
    Private m_StartImage As Image

    Private m_EditBitmap As Bitmap

    Private m_Ctl As IEditColorsControl
    Private m_ResetAction As Boolean = False

    Private m_EditColorValues As EditColorValues
    Private m_StartEditColorValues As EditColorValues



    Public Sub New(ByVal Ctl As IEditColorsControl, ByVal EditColorValues As EditColorValues)

        ' Dieser Aufruf ist fr den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fgen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_Ctl = Ctl
        m_BaseImage = Ctl.BaseImage
        m_StartImage = Ctl.Image
        m_EditColorValues = EditColorValues
        m_StartEditColorValues = EditColorValues.CreateClone
        m_EditBitmap = New Bitmap(m_BaseImage)

        m_ResetAction = True
        Me.TrackBarBrightness.Value = m_EditColorValues.Brightness
        Me.TrackBarContrast.Value = m_EditColorValues.Contrast
        Me.TrackBarGamma.Value = m_EditColorValues.Gamma
        Me.TrackBarHue.Value = m_EditColorValues.Hue
        Me.TrackBarSaturation.Value = m_EditColorValues.Saturation

        m_ResetAction = False
    End Sub

    Private Sub CalculateColors()
        'This code somewhat duplicates the individual transformations
        'But remember this is pedagocical code
        'In reality you would code things either as individual transformations
        'or in this combined mode - never the two in the same package.
        Dim MyMatrix As ColorMatrix

        Dim Brightness As Single = CSng(TrackBarBrightness.Value - 50) / 50
        Dim BrightnessMatrix As New ColorMatrix _
        (New Single()() _
        {New Single() {1, 0, 0, 0, 0}, _
        New Single() {0, 1, 0, 0, 0}, _
        New Single() {0, 0, 1, 0, 0}, _
        New Single() {0, 0, 0, 1, 0}, _
        New Single() {Brightness, Brightness, Brightness, 0, 1}})

        Dim Contrast As Single = CSng(TrackBarContrast.Value) / 50
        Const Minimum As Single = 0.0001
        Dim ContrastMatrix As New ColorMatrix _
        (New Single()() _
        {New Single() {Contrast, 0, 0, 0, 0}, _
        New Single() {0, Contrast, 0, 0, 0}, _
        New Single() {0, 0, Contrast, 0, 0}, _
        New Single() {0, 0, 0, 1, 0}, _
        New Single() {Minimum, Minimum, Minimum, 0, 1}})

        Dim Saturation As Single = CSng(TrackBarSaturation.Value / 50)
        If Saturation > 1 Then Saturation += 3 * (Saturation - 1)
        Dim SInv As Single = 1 - Saturation
        Dim SR As Single = CSng(0.309 * SInv)
        Dim SG As Single = CSng(0.609 * SInv)
        Dim SB As Single = CSng(0.082 * SInv)
        Dim SaturationMatrix As New ColorMatrix _
        (New Single()() _
        {New Single() {SR + Saturation, SR, SR, 0, 0}, _
        New Single() {SG, SG + Saturation, SG, 0, 0}, _
        New Single() {SB, SB, SB + Saturation, 0, 0}, _
        New Single() {0, 0, 0, 1, 0}, _
        New Single() {0, 0, 0, 0, 1}})


        Dim Hue As Single = CSng(TrackBarHue.Value - 50) * 180 / 50
        Hue *= CSng(Math.PI / 180)
        Dim HueMatrix As New ColorMatrix _
        (New Single()() _
        {New Single() {CSng(Math.Cos(Hue)), CSng(Math.Sin(Hue)), 0, 0, 0}, _
        New Single() {-CSng(Math.Sin(Hue)), CSng(Math.Cos(Hue)), 0, 0, 0}, _
        New Single() {0, 0, 1, 0, 0}, _
        New Single() {0, 0, 0, 1, 0}, _
        New Single() {0, 0, 0, 0, 1}})

        'Now create the total transform
        'by multiplying the individual terms together
        'The order of applying the B and C Matrices makes a difference
        'but I don't know which is 'right'
        MyMatrix = MatrixMultiply(BrightnessMatrix, ContrastMatrix)
        MyMatrix = MatrixMultiply(MyMatrix, SaturationMatrix)
        MyMatrix = MatrixMultiply(MyMatrix, HueMatrix)
        ApplyColorMatrix(MyMatrix)
    End Sub


    Private Function MatrixMultiply(ByVal MatrixA As ColorMatrix, ByVal MatrixB As ColorMatrix) As ColorMatrix
        'This multiplies two ColorMatrices because Microsoft didn't provide the function
        'that would have enhanced the power of ColorMatrix
        Dim MatrixResult As New ColorMatrix

        Dim Werte(4) As Single
        Dim Summe As Single

        For I As Integer = 0 To 4
            For J As Integer = 0 To 4
                Werte(J) = MatrixA(J, I)
            Next
            For K As Integer = 0 To 4
                Summe = 0
                For J As Integer = 0 To 4
                    Summe += MatrixB(K, J) * Werte(J)
                Next
                MatrixResult(K, I) = Summe
            Next
        Next
        Return MatrixResult
    End Function

    Private Sub ApplyColorMatrix(ByRef MyMatrix As ColorMatrix)
        'The magic that converts the image
        Dim ImgAtt As New ImageAttributes
        m_EditBitmap = New Bitmap(m_BaseImage.Width, m_BaseImage.Height)
        Dim grMatrix As Graphics = Graphics.FromImage(m_EditBitmap)
        'Put the matrix into an ImageAttribute
        ImgAtt.SetColorMatrix(MyMatrix)

        'This Gamma code doesn't fit in elegantly with the rest of the code
        'but it's a useful extra idea with very little code.
        'You would integrate it better in real code.
        'We want the Gamma value to vary from 0 to 4 with 1=normal
        Dim Gamma As Single = CSng(TrackBarGamma.Value) / 50
        If Gamma > 1 Then Gamma += 3 * (Gamma - 1)
        ImgAtt.SetGamma(Gamma)

        'Now draw the whole of MyImage onto bmpMatrix using the ImgAtt matrix
        grMatrix.DrawImage(m_BaseImage, New Rectangle(0, 0, m_BaseImage.Width, m_BaseImage.Height), 0, 0, m_BaseImage.Width, m_BaseImage.Height, GraphicsUnit.Pixel, ImgAtt)
        'Now show the transformed image
        m_Ctl.SetTemporaryImage(m_EditBitmap)
        'MyImage = bmpMatrix
        'Always a good habit to dispose of these graphic things
        grMatrix.Dispose()
        ImgAtt.Dispose()
    End Sub

    Private Sub ButtonReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonReset.Click
        m_ResetAction = True
        TrackBarBrightness.Value = 50
        TrackBarContrast.Value = 50
        TrackBarSaturation.Value = 50
        TrackBarGamma.Value = 50
        TrackBarHue.Value = 50

        m_EditColorValues.Reset()

        m_Ctl.SetTemporaryImage(m_BaseImage)

        m_ResetAction = False
    End Sub

 

    Private Sub TrackBarBrightness_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBarBrightness.Scroll
        If m_ResetAction Then Exit Sub
        m_EditColorValues.Brightness = TrackBarBrightness.Value
        CalculateColors()
    End Sub

    Private Sub TrackBarContrast_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBarContrast.Scroll
        If m_ResetAction Then Exit Sub
        m_EditColorValues.Contrast = TrackBarContrast.Value
        CalculateColors()
    End Sub

    Private Sub TrackBarSaturation_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBarSaturation.Scroll
        If m_ResetAction Then Exit Sub
        m_EditColorValues.Saturation = TrackBarSaturation.Value
        CalculateColors()
    End Sub

    Private Sub TrackBarGamma_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBarGamma.Scroll
        If m_ResetAction Then Exit Sub
        m_EditColorValues.Gamma = TrackBarGamma.Value
        CalculateColors()
    End Sub

    Private Sub TrackBarHue_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackBarHue.Scroll
        If m_ResetAction Then Exit Sub
        m_EditColorValues.Hue = TrackBarHue.Value
        CalculateColors()
    End Sub

    Private Sub frm_EditColors_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Me.DialogResult <> Windows.Forms.DialogResult.OK Then
            m_Ctl.SetTemporaryImage(m_StartImage)
            m_Ctl.EditColorValues = m_StartEditColorValues
        End If
    End Sub
End Class